<?php
if (!defined('ABSPATH'))
    exit;

use Automattic\WooCommerce\Internal\DataStores\Orders\CustomOrdersTableController;

add_action('add_meta_boxes', 'lar_order_meta_box');

if (!function_exists('lar_order_meta_box')) {
    /**
     * Adds a meta box displaying the order's services and quotes.
     * @return void
     */
    function lar_order_meta_box() {
        try {
            if (class_exists('\Automattic\WooCommerce\Internal\DataStores\Orders\CustomOrdersTableController') &&
                wc_get_container()->get(CustomOrdersTableController::class)->custom_orders_table_usage_is_enabled())
                $screen = wc_get_page_screen_id('shop-order');
            else
                $screen = 'shop_order';
        } catch (\Throwable $e) {
            $screen = 'shop_order';
        }

        add_meta_box(
            'lar_order_meta_box',
            esc_html__('Ship Discounts', 'ship-discounts-lar'),
            'lar_order_meta_box_callback',
            $screen, //'woocommerce_page_wc-orders',
            'side',
            'high'
        );
    }
}

if (!function_exists('lar_order_meta_box_callback')) {
    /**
     * Callback function for the Ship Discounts meta box.
     * @return void
     */
    function lar_order_meta_box_callback($post) {
        if (get_option("lar_api_dev", "")) {
            echo '<p class="lar_danger_message"><strong>' . esc_html__('Be careful! Development mode is activated. No real orders will be created.', 'ship-discounts-lar') . '</strong></p>';
        }

        $order = wc_get_order($post->ID);
        if ($order) {
            if (!$order->has_shipping_method('lar_method'))
                echo '<p>' . esc_html__('Note: the customer did not choose the Ship Discounts shipping method.', 'ship-discounts-lar') . '</p>';

            if (!lar_check_shop_info())
                /* translators: %s: WooCommerce general settings page */
                echo '<p class="lar_danger_message">' . sprintf(__('You must enter <a target="_blank" href="%s">your store\'s contact details</a> in order to create an order.', 'ship-discounts-lar'), admin_url('admin.php?page=wc-settings&tab=general')) . '</p>'; // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
            else if (!$order->get_meta('_lar_order_number'))
                echo '<p>' . esc_html__('The order has not been submitted yet.', 'ship-discounts-lar') . '</p>';
            else {
                $order_number = $order->get_meta('_lar_order_number');
                $carrier_code = $order->get_meta('_lar_carrier_code');
                $service_code = $order->get_meta('_lar_service_code');
                $cost = $order->get_meta('_lar_service_cost');
                $display_cost = $order->get_meta('_lar_service_display_cost') !== null && $order->get_meta('_lar_service_display_cost') !== '' ? $order->get_meta('_lar_service_display_cost') : $cost;
                $signature = $order->get_meta('_lar_signature') ? esc_html__('The signature is required.', 'ship-discounts-lar') : '';
                $ncv = $order->get_meta('_lar_ncv') ? esc_html__('There is a non-conveyable object.', 'ship-discounts-lar') : '';

                echo '<p>' . esc_html__('Order number', 'ship-discounts-lar') . ': <b>' . esc_attr($order_number) . '</b>';
                echo '<br>' . esc_html__('Carrier code', 'ship-discounts-lar') . ': <b>' . esc_attr($carrier_code) . '</b>';
                echo '<br>' . esc_html__('Service code', 'ship-discounts-lar') . ': <b>' . esc_attr($service_code) . '</b>';
                // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
                echo '<br>' . esc_html__('Quoted rate', 'ship-discounts-lar') . ': <b>' . wc_price($cost) . '</b>';
                // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
                echo '<br>' . esc_html__('Shown rate', 'ship-discounts-lar') . ': <b>' . wc_price($display_cost) . '</b></p>';

                if ($signature || $ncv) {
                    echo '<p>';
                    // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
                    echo $signature ? $signature . '<br>' : '';
                    // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
                    echo $ncv;
                    echo '</p>';
                }

                echo '<p>';
                // Tracking & Status
                $details = wc_clean(Ship_Discounts_Carriers::getOrder($order_number, $order));
                if ($details) {
                    if (!$order->get_meta('_lar_tracking_nb'))
                        $order->update_meta_data('_lar_tracking_nb', $details['tracking']);
                    if (!$order->get_meta('_lar_tracking_url'))
                        $order->update_meta_data('_lar_tracking_url', $details['trackingUrl']);
                    $order->save();
                }
                $tracking = $order->get_meta('_lar_tracking_nb');
                $tracking_url = $order->get_meta('_lar_tracking_url');

                if (!$details)
                    echo esc_html__('Sorry, we can not fetch the order\'s current status right now.', 'ship-discounts-lar') . '<br>';
                else if ($details['CarrierDeliveryStatus']) {
                    // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
                    echo esc_html__('Delivery status:', 'ship-discounts-lar') . ' <b>' . Ship_Discounts_Carriers::getDeliveryStatus()[$details['CarrierDeliveryStatus']] . '</b><br>';
                }

                if ($tracking) {
                    echo esc_html__('Tracking:', 'ship-discounts-lar') . ' <b><a href="' . esc_url($tracking_url) . '" target="_blank">' . esc_attr($tracking) . '</a></b><br>';
                }

                // Label
                if (!$order->get_meta('_lar_label')) {
                    $label = wc_clean(Ship_Discounts_Carriers::getLabel($order_number, $order));
                    $order->update_meta_data('_lar_label', $label === false ? false : $label[0]);
                    $order->save();
                }
                $label = $order->get_meta('_lar_label');

                if ($label === false) {
                    echo esc_html__('Sorry, we can not fetch the order\'s label right now.', 'ship-discounts-lar') . '<br>';
                }
                else {
                    if ($label) {
                        echo '<b><a href="' . esc_url($label) . '" target="_blank">' . esc_html__('See the label', 'ship-discounts-lar') . '</a></b>';
                    }
                    else
                        echo esc_html__('The label has not been created yet.', 'ship-discounts-lar');
                }
                echo '</p>';
            }

            // Client URL
            echo '<p><b><a href="' . esc_url(get_option("lar_client_url", "https://client.montrealdropship.com/")) . '" target="_blank">' .
                esc_html__('Manage my orders (Customer Portal)', 'ship-discounts-lar') . '</a></b></p>';

            // (Re)send order
            if ((!$order->get_meta('_lar_order_number') || LAR_ADVANCED_SETTINGS) && lar_check_shop_info())
                lar_send_order_modal($order);

            // Previous not deleted orders
            if (LAR_ADVANCED_SETTINGS) {
                $old_orders = $order->get_meta('_lar_old_orders');
                if ($old_orders) {
                    echo '<p class="lar_danger_message"><strong>' .
                        esc_html__('These previous orders have not been cancelled. You should remove them.', 'ship-discounts-lar') .
                        '</strong><br>';
                    foreach ($old_orders as $old_order)
                        echo '#' . esc_attr($old_order) . '<br>';
                    echo '</p>';

                    wp_nonce_field('lar-ajax-nonce-del', 'lar-nonce-del', true, false);

                    echo '<button id="btn-lar-cancel-orders" name="btn-lar-cancel-orders" class="button" type="button" value="cancel">' .
                        esc_html__('Cancel these orders', 'ship-discounts-lar') .
                        '</button>';

                    ob_start(); ?>
                    <script>
                        jQuery(function ($) {
                            $("#btn-lar-cancel-orders").on("click", function (e) {
                                e.preventDefault();
                                $(document.body).css({'cursor': 'progress'});

                                $.ajax({
                                    type: 'POST',
                                    url: '<?php echo esc_url(admin_url('admin-ajax.php')) ?>',
                                    data: {
                                        'action': 'lar_cancel_orders',
                                        'nonce': $('#lar-nonce-del'),
                                        'order': '<?php echo esc_attr($order->get_id()) ?>',
                                    },
                                    success: function (result) {
                                        $(document.body).css({'cursor': 'default'});
                                        let data = $.parseJSON(result);
                                        let button = $('#btn-lar-cancel-orders');
                                        let text = button.prev('.lar_danger_message');

                                        if (data !== false) {
                                            if (data.length === 0) {
                                                text.remove();
                                                button.remove();
                                            } else {
                                                let html = "<strong>\
                                                <?php echo esc_html__('These orders could not be cancelled. Please try again.', 'ship-discounts-lar') ?>\
                                                </strong><br>";
                                                $.each(data, function (i, v) {
                                                    html += '#' + v + '<br>';
                                                });
                                                text.html(html);
                                            }
                                        } else {
                                            text.find('strong').html("<?php echo esc_html__('An error has occurred. Please try again.', 'ship-discounts-lar') ?>");
                                        }
                                    },
                                    error: function (error) {
                                        $(document.body).css({'cursor': 'default'});
                                        $('#btn-lar-cancel-orders').prev('.lar_danger_message').find('strong').html("<?php echo esc_html__('An error has occurred. Please try again.', 'ship-discounts-lar') ?>");
                                    }
                                });
                            });
                        });
                    </script>
                    <?php
                    // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
                    echo ob_get_clean();
                }
            }
        }
    }
}

if (!function_exists('lar_send_order_modal')) {
    /**
     * Displays the HTML of the "Send to Ship Discounts" modal.
     * @param $order WC_Order|WC_Order_Refund Order.
     * @return void
     */
    function lar_send_order_modal($order) {
        add_thickbox();
        ob_start(); ?>

        <br>
        <a href="#TB_inline?&width=600&height=550&inlineId=lar-send-order"
           title="<?php echo esc_html__('Send to Ship Discounts', 'ship-discounts-lar') ?>"
           class="thickbox button">
            <?php echo esc_html__('Send to Ship Discounts', 'ship-discounts-lar') ?>
        </a>

        <div id="lar-send-order" style="display:none;">
            <p class="send-order-desc">
                <?php echo esc_html__('You can (re)send an order for these products to this address. You can change the carrier, packaging, boxes cost, whether a signature is required and if there are one or more non-conveyable products. Please note that the total cost may change.', 'ship-discounts-lar') ?>
            </p>

            <form id="lar-send-order-form">
                <table class="form-table">
                    <tbody>
                    <tr style="vertical-align:top">
                        <th scope="row" class="titledesc">
                            <label for="signature">
                                <?php echo esc_html__('Signature', 'ship-discounts-lar') ?>
                                <?php
                                // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
                                echo WC_Admin_Settings::get_field_description([
                                        'desc_tip' => true,
                                        'desc'     => esc_html__('Whether or not the signature is required.', 'ship-discounts-lar')]
                                )['tooltip_html']; ?>
                            </label>
                        </th>
                        <td class="forminp forminp-text">
                            <input name="signature" id="signature" type="checkbox"
                                   value="1" <?php echo $order->get_meta('_lar_signature') ? 'checked' : '' ?>>
                        </td>
                    </tr>

                    <tr style="vertical-align:top">
                        <th scope="row" class="titledesc">
                            <label for="ncv">
                                <?php echo esc_html__('Non-conveyable', 'ship-discounts-lar') ?>
                                <?php
                                // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
                                echo WC_Admin_Settings::get_field_description([
                                        'desc_tip' => true,
                                        'desc'     => esc_html__('Whether or not this contains a non-conveyable product.', 'ship-discounts-lar')]
                                )['tooltip_html']; ?>
                            </label>
                        </th>
                        <td class="forminp forminp-text">
                            <input name="ncv" id="ncv" type="checkbox"
                                   value="1" <?php echo $order->get_meta('_lar_ncv') ? 'checked' : '' ?>>
                        </td>
                    </tr>
                    </tbody>
                </table>

                <table>
                    <tbody>
                    <tr style="vertical-align:top">
                        <th scope="row" class="titledesc">
                            <label>
                                <?php echo esc_html__('Package dimensions and weights', 'ship-discounts-lar') ?>
                                <?php
                                // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
                                echo WC_Admin_Settings::get_field_description([
                                        'desc_tip' => true,
                                        'desc'     => esc_html__('Dimensions and weights of the used packages.', 'ship-discounts-lar')]
                                )['tooltip_html']; ?>
                            </label>
                        </th>
                    </tr>
                    <tr style="vertical-align:top">
                        <td>
                            <table class="lar_boxes_table widefat">
                                <thead>
                                <tr>
                                    <th><input type="checkbox"/></th>
                                    <th><?php echo esc_html__('Length', 'ship-discounts-lar'); ?></th>
                                    <th><?php echo esc_html__('Width', 'ship-discounts-lar'); ?></th>
                                    <th><?php echo esc_html__('Height', 'ship-discounts-lar'); ?></th>
                                    <th><?php echo esc_html__('Weight', 'ship-discounts-lar'); ?></th>
                                </tr>
                                </thead>
                                <tfoot>
                                <tr>
                                    <th colspan="13">
                                        <a href="#"
                                           class="button lar_boxes_insert"><?php echo esc_html__('Add box', 'ship-discounts-lar'); ?></a>
                                        <a href="#"
                                           class="button lar_boxes_remove"><?php echo esc_html__('Remove selected box(es)', 'ship-discounts-lar'); ?></a>
                                    </th>
                                </tr>
                                </tfoot>
                                <tbody>
                                <?php
                                $packages = $order->get_meta('_lar_packages');
                                if ($packages) {
                                    foreach ($packages as $key => $box) { ?>
                                        <tr>
                                            <td class="check-column"><span><input type="checkbox"/></span></td>
                                            <td><span><input type="text" name="packages_length[<?php echo esc_attr($key); ?>]"
                                                             value="<?php echo esc_attr($box['length']); ?>"/>in</span>
                                            </td>
                                            <td><span><input type="text" name="packages_width[<?php echo esc_attr($key); ?>]"
                                                             value="<?php echo esc_attr($box['width']); ?>"/>in</span>
                                            </td>
                                            <td><span><input type="text" name="packages_height[<?php echo esc_attr($key); ?>]"
                                                             value="<?php echo esc_attr($box['height']); ?>"/>in</span>
                                            </td>
                                            <td><span><input type="text" name="packages_weight[<?php echo esc_attr($key); ?>]"
                                                             value="<?php echo esc_attr($box['weight']); ?>"/>lbs</span>
                                            </td>
                                        </tr>
                                        <?php
                                    }
                                } ?>
                                </tbody>
                            </table>
                        </td>
                    </tr>
                    </tbody>
                </table>

                <table class="form-table">
                    <tbody>
                    <tr style="vertical-align:top">
                        <th scope="row" class="titledesc">
                            <label for="cost-boxes">
                                <?php echo esc_html__('Extra cost of box(es)', 'ship-discounts-lar') ?>
                                <?php
                                // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
                                echo WC_Admin_Settings::get_field_description([
                                        'desc_tip' => true,
                                        'desc'     => esc_html__('Additional cost for using these boxes. Will be added to shipping costs.', 'ship-discounts-lar')]
                                )['tooltip_html']; ?>
                            </label>
                        </th>
                        <td class="forminp forminp-text">
                            <input name="cost-boxes" id="cost-boxes" type="text"
                                   value="<?php echo esc_attr($order->get_meta('_lar_boxes_cost')) ?: 0 ?>">
                        </td>
                    </tr>
                    </tbody>
                </table>

                <table class="form-table">
                    <tbody>
                    <tr style="vertical-align:top">
                        <th scope="row" class="titledesc">
                            <label>
                                <?php echo esc_html__('Carriers', 'ship-discounts-lar') ?>
                                <?php
                                // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
                                echo WC_Admin_Settings::get_field_description([
                                        'desc_tip' => true,
                                        'desc'     => esc_html__('Chosen carrier.', 'ship-discounts-lar')]
                                )['tooltip_html']; ?>
                            </label>
                        </th>
                        <td class="forminp forminp-text">
                            <button id="btn-lar-get-quotes" name="btn-lar-get-quotes"
                                    class="button" type="button" value="quotes">
                                <?php echo esc_html__('Get quotes', 'ship-discounts-lar') ?>
                            </button>
                        </td>
                    </tr>
                    </tbody>
                </table>

                <?php
                $carrier_data = $order->get_meta('lar_carrier_data');

                $carrier_code = $order->get_meta('_lar_carrier_code');
                $service_code = $order->get_meta('_lar_service_code');
                $cost = $order->get_meta('_lar_service_cost');
                $display_cost = $order->get_meta('_lar_service_display_cost') !== null && $order->get_meta('_lar_service_display_cost') !== '' ? $order->get_meta('_lar_service_display_cost') : $cost;

                if ($carrier_data) {
                    if (!$carrier_code) $carrier_code = $carrier_data['carrierCode'];
                    if (!$service_code) $service_code = $carrier_data['services']['serviceCode'];
                    if (!$cost) $cost = $carrier_data['services']['cost'];
                    if ($display_cost == null) $display_cost = $carrier_data['services']['displayCost'] !== null && $carrier_data['services']['displayCost'] !== '' ? $carrier_data['services']['displayCost'] : $cost;
                }

                $sku = true;
                foreach ($order->get_items() as $item) {
                    $sku = $item->get_product()->get_sku() ?: "";
                    if (!$sku) break;
                }
                ?>

                <div class="carriers-list">
                    <div class="carriers-list-header">
                        &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
                        <span><?php echo esc_html__('Carrier code', 'ship-discounts-lar') ?></span>
                        <span><?php echo esc_html__('Service code', 'ship-discounts-lar') ?></span>
                        <span><?php echo esc_html__('Quoted rate', 'ship-discounts-lar') ?></span>
                        <span><?php echo esc_html__('Shown rate', 'ship-discounts-lar') ?></span>
                    </div>
                    <div class="carriers-list-carriers">
                        <?php if ($carrier_code) { ?>
                            <div class="carriers-list-carrier default-carrier">
                                <input type="radio" id="default" name="carrier" value="default" checked>
                                <label for="default">
                                    <span><?php echo esc_attr($carrier_code) ?></span>
                                    <span><?php echo esc_attr($service_code) ?></span>
                                    <span><?php echo wc_price($cost) /* phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped */ ?></span>
                                    <span><?php echo wc_price($display_cost) /* phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped */ ?></span>
                                </label>
                            </div>
                        <?php } ?>
                    </div>
                </div>

                <p id="lar-error-msg" class="lar_danger_message" style="display:none"></p>

                <?php
                // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
                echo wp_nonce_field('lar-ajax-nonce', 'lar-nonce', true, false); ?>

                <p class="submit">
                    <button id="btn-lar-send-order" name="btn-lar-send-order"
                            class="button-primary" type="submit" value="submit">
                        <?php echo esc_html__('Send to Ship Discounts', 'ship-discounts-lar') ?>
                    </button>
                </p>
            </form>

            <script>
                jQuery(function ($) {
                    /**
                     * Select all boxes.
                     */
                    $('.lar_boxes_table thead input[type="checkbox"]').on('change', function (e) {
                        if ($(this).is(':checked'))
                            $('.lar_boxes_table').find('tbody').find('.check-column input[type="checkbox"]').prop('checked', true);
                        else
                            $('.lar_boxes_table').find('tbody').find('.check-column input[type="checkbox"]').prop('checked', false);
                    });

                    /**
                     * Remove all checked boxes from the table.
                     */
                    $('.lar_boxes_table .lar_boxes_remove').on('click', function (e) {
                        e.preventDefault();
                        $('.lar_boxes_table thead input[type="checkbox"]').prop('checked', false);

                        $('.lar_boxes_table').find('tbody').find('.check-column input:checked').each(function () {
                            $(this).closest('tr').remove();
                        });
                    });

                    /**
                     * Add a new row in the boxes table.
                     */
                    $('.lar_boxes_table .lar_boxes_insert').on('click', function (e) {
                        $('.lar_boxes_table thead input[type="checkbox"]').prop('checked', false);

                        let tbody = $('.lar_boxes_table').find('tbody');
                        let size = tbody.find('tr').size();
                        let code = '<tr class="new">\
									<td class="check-column"><span><input type="checkbox" /></span></td>\
									<td><span><input type="text" name="packages_length[' + size + ']" />in</span></td>\
									<td><span><input type="text" name="packages_width[' + size + ']" />in</span></td>\
									<td><span><input type="text" name="packages_height[' + size + ']" />in</span></td>\
									<td><span><input type="text" name="packages_weight[' + size + ']" />lbs</span></td>\
								</tr>';
                        tbody.append(code);
                    });

                    /**
                     * Get the carriers' quotes.
                     */
                    $('#btn-lar-get-quotes').on('click', function (e) {
                        e.preventDefault();
                        $('.carriers-list-carriers div').not('.default-carrier').remove();
                        $('.carriers-list-carriers input[type="radio"]').prop('checked', true);

                        if (!"<?php echo esc_attr($sku) ?>") {
                            let html = '<div class="carrier-loader">' + "<?php echo esc_html__('The products must have a SKU.', 'ship-discounts-lar') ?>" + '</div>';
                            $('.carriers-list-carriers').append(html);
                            return;
                        }

                        $('.carriers-list-carriers').append("<div class='carrier-loader'><img src='" + imgLoader.src + "'/></div>");

                        let signature = $('#signature').is(':checked') ? 1 : 0;
                        let ncv = $('#ncv').is(':checked') ? 1 : 0;
                        let packages = getPackages();

                        $.ajax({
                            type: 'POST',
                            url: '<?php echo esc_url(admin_url('admin-ajax.php')) ?>',
                            data: {
                                'action': 'lar_get_carriers_quotes',
                                'nonce': $('#lar-nonce').val(),
                                'order': '<?php echo esc_attr($order->get_id()) ?>',
                                'signature': signature,
                                'ncv': ncv,
                                'packages': packages,
                            },
                            success: function (result) {
                                $('.carriers-list-carriers div').not('.default-carrier').remove();
                                let data = $.parseJSON(result);
                                let html = '';

                                if (data === false) {
                                    html = '<div class="carrier-loader">' + "<?php echo esc_html__('Sorry, we could not fetch the carriers\' quotes. Please try again.', 'ship-discounts-lar') ?>" + '</div>';
                                } else if (data === "POSTALCODE") {
                                    html = '<div class="carrier-loader">' + "<?php echo esc_html__('To view our carriers\' quotes, please insert a valid postal code.', 'ship-discounts-lar') ?>" + '</div>';
                                } else if (data === "DIMENSIONS") {
                                    html = '<div class="carrier-loader">' + "<?php echo esc_html__('The products must have valid dimensions and weight.', 'ship-discounts-lar') ?>" + '</div>';
                                } else {
                                    // Carriers
                                    $.each(data, function (c_code, carrier) {
                                        $.each(carrier['services'], function (code, service) {
                                            let displayCost = "displayCost" in service ? service['displayCost'] : service['cost'];
                                            html += '<div class="carriers-list-carrier">\
                                                <input type="radio" id="' + c_code + '%' + code + '" name="carrier" value="' + c_code + '%' + code + '">\
                                                    <label for="' + c_code + '%' + code + '">\
                                                        <span>' + c_code + '</span>\
                                                        <span>' + code + '</span>\
                                                        <span>' + service['cost'] + '</span>\
                                                        <span>' + displayCost + '</span>\
                                                    </label>\
                                            </div>';
                                        });
                                    });
                                }

                                $('.carriers-list-carriers').append(html);
                            },
                            error: function (error) {
                                $('.carriers-list-carriers div').not('.default-carrier').remove();
                                let html = '<div class="carrier-loader">' + "<?php echo esc_html__('Sorry, we could not fetch the carriers\' quotes. Please try again.', 'ship-discounts-lar') ?>" + '</div>';
                                $('.carriers-list-carriers').append(html);
                            }
                        });
                    });

                    /**
                     * Send to Ship Discounts.
                     */
                    $('#btn-lar-send-order').on('click', function (e) {
                        e.preventDefault();
                        $(document.body).css({'cursor': 'progress'});
                        $('#lar-error-msg').hide();

                        let signature = $('#signature').is(':checked') ? 1 : 0;
                        let ncv = $('#ncv').is(':checked') ? 1 : 0;
                        let cost_boxes = $('#cost-boxes').val();
                        let packages = getPackages();

                        let carrier_code = 'default';
                        let service_code = 'default';
                        let cost = 'default';
                        let display_cost = 'default';

                        let val = $('.carriers-list-carrier input[name=carrier]:checked').val();
                        if (val != null && val !== 'default') {
                            let codes = val.split('%');
                            if (codes.length > 0) {
                                carrier_code = codes[0];
                                service_code = codes[1];

                                let count = 0;
                                $('label[for="' + val + '"] span').each(function () {
                                    if (count === 3)
                                        cost = $(this).text().replace(/[^0-9.,]/g, '');
                                    if (count === 5)
                                        display_cost = $(this).text().replace(/[^0-9.,]/g, '');
                                    count++;
                                });
                            }
                        } else if (val == null) {
                            carrier_code = '';
                            service_code = '';
                            cost = '';
                            display_cost = '';
                        }

                        $.ajax({
                            type: 'POST',
                            url: '<?php echo esc_url(admin_url('admin-ajax.php')) ?>',
                            data: {
                                'action': 'lar_resend_order',
                                'nonce': $('#lar-nonce').val(),
                                'order': '<?php echo esc_attr($order->get_id()) ?>',
                                'signature': signature,
                                'ncv': ncv,
                                'packages': packages,
                                'cost_boxes': cost_boxes,
                                'carrier_code': carrier_code,
                                'service_code': service_code,
                                'cost': cost,
                                'display_cost': display_cost,
                            },
                            success: function (result) {
                                $(document.body).css({'cursor': 'default'});
                                let data = $.parseJSON(result);

                                if (data === true)
                                    location.reload();
                                else {
                                    $('#lar-error-msg').show();
                                    if (data === 'DELETE') {
                                        <?php /* translators: %s: Ship Discounts website */ ?>
                                        $('#lar-error-msg').html("<strong><?php echo sprintf(__('The new order was not created because the current order could not be cancelled.<br><a target=\'_blank\' href=\'%s\'>You can try to delete it manually.</a>', 'ship-discounts-lar'), esc_url(get_option("lar_client_url", "https://client.montrealdropship.com/"))) /* phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped */ ?></strong>");
                                    }
                                    else if (data === 'PHONE')
                                        $('#lar-error-msg').html("<strong><?php echo esc_html__('The order was not created because the shop phone number is invalid. Please note that extensions will not be used.', 'ship-discounts-lar') ?></strong>");
                                    else if (data === 'MISSING')
                                        $('#lar-error-msg').html("<strong><?php echo esc_html__('The order was not created because one or more mandatory fields are missing. You can start by checking the customer\'s shipping address.', 'ship-discounts-lar') ?></strong>");
                                    else if (data === 'PACKAGES')
                                        $('#lar-error-msg').html("<strong><?php echo esc_html__('The order was not created because there a no packages.', 'ship-discounts-lar') ?></strong>");
                                    else
                                        $('#lar-error-msg').html("<strong><?php echo esc_html__('The order could not be created. Please try again.', 'ship-discounts-lar') ?></strong>");
                                }
                            },
                            error: function (error) {
                                $(document.body).css({'cursor': 'default'});
                                $('#lar-error-msg').show();
                                $('#lar-error-msg').html("<strong><?php echo esc_html__('The order could not be created. Please try again.', 'ship-discounts-lar') ?></strong>");
                            }
                        });
                    });

                    /**
                     * Check if a value is numeric.
                     * @param n Value.
                     * @returns {boolean} If the value is numeric.
                     */
                    function isNumeric(n) {
                        return !isNaN(parseFloat(n)) && isFinite(n);
                    }

                    /**
                     * Get the packages from the table.
                     * @returns {*[]} Packages.
                     */
                    function getPackages() {
                        let packages = [];
                        let i = 0;

                        $('.lar_boxes_table tbody tr').each(function () {
                            let length = $(this).find('input[name^=packages_length]').val();
                            let width = $(this).find('input[name^=packages_width]').val();
                            let height = $(this).find('input[name^=packages_height]').val();
                            let weight = $(this).find('input[name^=packages_weight]').val();

                            if (!isNumeric(length)) length = 0;
                            if (!isNumeric(width)) width = 0;
                            if (!isNumeric(height)) height = 0;
                            if (!isNumeric(weight)) weight = 0;

                            packages[i] = {
                                'length': length,
                                'width': width,
                                'height': height,
                                'weight': weight,
                            };
                            i++;
                        });

                        return packages;
                    }
                });
            </script>
        </div>

        <?php
        // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
        echo ob_get_clean();
    }
}

if (!function_exists('lar_check_shop_info')) {
    /**
     * Checks if the shop has all the information needed for the order creation.
     * @return bool
     */
    function lar_check_shop_info() {
        return get_bloginfo('name') && WC()->countries->get_base_address() &&
            str_replace(' ', '', WC()->countries->get_base_postcode()) &&
            get_option('lar_settings_phone_number', '') && WC()->countries->get_base_city();
    }
}

if (!function_exists('prefix_ajax_lar_get_carriers_quotes')) {
    /**
     * Returns the carriers' quotes.
     * @return void
     */
    function prefix_ajax_lar_get_carriers_quotes() {
        if (isset($_POST['order']) && wp_verify_nonce($_POST['nonce'], 'lar-ajax-nonce') &&
            $_POST['order'] && current_user_can('edit_posts')) {
            $order = wc_get_order($_POST['order']);

            if ($order) {
                $signature = isset($_POST['signature']) && $_POST['signature'];
                $ncv = isset($_POST['ncv']) && $_POST['ncv'];
                $postcode = $order->get_shipping_postcode() ?: '';

                $shipping_methods = $order->get_shipping_methods();
                if ($shipping_methods) {
                    $shipping_method = reset($shipping_methods);
                    $shipping_class_names = WC()->shipping->get_shipping_method_class_names();
                    if (array_key_exists($shipping_method->get_method_id(), $shipping_class_names))
                        $method = new $shipping_class_names[$shipping_method->get_method_id()]($shipping_method->get_instance_id());
                }

                if (!isset($method)) {
                    $method = new WC_Shipping_Ship_Discounts();
                }

                $packages = [];
                if (isset($_POST['packages']) && is_array($_POST['packages'])) $packages = $_POST['packages'];

                $free_shipping = apply_filters('lar_free_shipping_value', get_option('lar_settings_free_shipping_value', ''), $order->get_user(), $order->get_items());
                $free_shipping = is_numeric($free_shipping) && floatval(($order->get_subtotal())) >= floatval($free_shipping);

                $carriers = wc_clean(Ship_Discounts_Carriers::getQuotes($method, $postcode, $packages, $signature, $ncv, false, false, $free_shipping, $order->get_user(), $order->get_items()));

                // Format price
                if (is_array($carriers)) {
                    foreach ($carriers as $c_code => $carrier) {
                        foreach ($carrier['services'] as $code => $service) {
                            $carriers[$c_code]['services'][$code]['cost'] = wc_price($service['cost']);
                            if (isset($service['displayCost']) && $service['displayCost'] !== '')
                                $carriers[$c_code]['services'][$code]['displayCost'] = wc_price($service['displayCost']);
                            else
                                $carriers[$c_code]['services'][$code]['displayCost'] = $carriers[$c_code]['services'][$code]['cost'];
                        }
                    }
                }

                die(wp_json_encode($carriers));
            }
        }
        die(wp_json_encode(false));
    }

    add_action('wp_ajax_lar_get_carriers_quotes', 'prefix_ajax_lar_get_carriers_quotes');
}

if (!function_exists('prefix_ajax_lar_resend_order')) {
    /**
     * Update and resend the order.
     * @return void
     */
    function prefix_ajax_lar_resend_order() {
        if (isset($_POST['order']) && wp_verify_nonce($_POST['nonce'], 'lar-ajax-nonce') &&
            $_POST['order'] && current_user_can('edit_posts')) {
            $order = wc_get_order($_POST['order']);

            if ($order) {
                $signature = isset($_POST['signature']) && $_POST['signature'];
                $ncv = isset($_POST['ncv']) && $_POST['ncv'];
                $cost_boxes = isset($_POST['cost_boxes']) ? floatval($_POST['cost_boxes']) : 0;

                $packages = [];
                if (isset($_POST['packages']) && is_array($_POST['packages'])) $packages = wc_clean($_POST['packages']);
                if (!$packages)
                    die(wp_json_encode('PACKAGES'));

                $carrier_code = wc_clean($_POST['carrier_code']) ?? '';
                $service_code = wc_clean($_POST['service_code']) ?? '';
                $cost = floatval($_POST['cost']) ?? 0;
                $display_cost = floatval($_POST['display_cost']) ?? 0;

                $shipping_methods = $order->get_shipping_methods();
                if ($shipping_methods) {
                    $shipping_method = reset($shipping_methods);
                    $shipping_class_names = WC()->shipping->get_shipping_method_class_names();
                    if (array_key_exists($shipping_method->get_method_id(), $shipping_class_names))
                        $method = new $shipping_class_names[$shipping_method->get_method_id()]($shipping_method->get_instance_id());
                }

                if (!isset($method)) {
                    $method = new WC_Shipping_Ship_Discounts();
                }

                $carrier = [];
                if ($carrier_code === 'default' || $service_code === 'default') {
                    $carrier = $order->get_meta('lar_carrier_data');
                    $display_cost = $carrier['services']['displayCost'] !== null && $carrier['services']['displayCost'] !== '' ? $carrier['services']['displayCost'] : $carrier['services']['cost'];
                }
                else {
                    $carrier['insurance'] = $method->shipment_value;
                    $carrier['carrierCode'] = $carrier_code;
                    $carrier['carrierName'] = $method->carriers_list[$carrier_code]['name'];
                    $carrier['services']['serviceCode'] = $service_code;
                    $carrier['services']['cost'] = $cost;
                    $carrier['services']['displayCost'] = $display_cost;
                }

                // Delete previous order
                $ok_create = true;
                $old_code = $order->get_meta('_lar_order_number');
                if ($old_code) {
                    $deleted = Ship_Discounts_Carriers::deleteOrder($old_code, $order);
                    if ($deleted === false || (isset($deleted['result']) && $deleted['result'] != 'success'))
                        $ok_create = false;

                    // TODO: Gérer les cas d'erreur
                }

                if ($ok_create) {
                    $code = wc_clean(Ship_Discounts_Carriers::submitOrder($order, $carrier, $packages, $signature, $ncv));
                    if ($code !== false && is_numeric($code)) {
                        // Update metadata
                        $order->update_meta_data('lar_carrier_data', $carrier);
                        $order->update_meta_data('_lar_signature', $signature);
                        $order->update_meta_data('_lar_ncv', $ncv);
                        $order->update_meta_data('_lar_packages', $packages);
                        $order->update_meta_data('_lar_boxes_cost', $cost_boxes);

                        $order->update_meta_data('_lar_order_number', $code);
                        $order->update_meta_data('_lar_carrier_code', $carrier['carrierCode']);
                        $order->update_meta_data('_lar_service_code', $carrier['services']['serviceCode']);
                        $order->update_meta_data('_lar_service_cost', $carrier['services']['cost']);
                        if ($carrier['services']['displayCost'] !== null && $carrier['services']['displayCost'] !== '')
                            $order->update_meta_data('_lar_service_display_cost', $carrier['services']['displayCost']);

                        $label = wc_clean(Ship_Discounts_Carriers::getLabel($code, $order));
                        if ($label)
                            $order->update_meta_data('_lar_label', $label[0]);

                        $details = wc_clean(Ship_Discounts_Carriers::getOrder($code, $order));
                        if ($details) {
                            $order->update_meta_data('_lar_tracking_url', $details['trackingUrl']);
                            $order->update_meta_data('_lar_tracking_nb', $details['tracking']);
                        }

                        // Update shipping cost
                        $free_shipping = apply_filters('lar_free_shipping_value', get_option('lar_settings_free_shipping_value', ''), $order->get_user(), $order->get_items());
                        $free_shipping = is_numeric($free_shipping) && floatval(($order->get_subtotal())) >= floatval($free_shipping);

                        if (!$order->get_items('shipping')) {
                            $item = new WC_Order_Item_Shipping();
                            $item->set_method_title(esc_html__('Ship Discounts', 'ship-discounts-lar'));
                            $item->set_method_id("lar_method");
                            $order->add_item($item);
                            $order->save();
                        }

                        foreach ($order->get_items('shipping') as $item) {
                            if ($free_shipping) $item->set_total(0);
                            else {
                                $method_cost = $method->cost ?: 0;
                                $item->set_total(floatval(apply_filters('lar_method_cost_value', $method_cost, $order->get_user(), $order->get_items())) + $display_cost + $cost_boxes);
                            }
                            $item->calculate_taxes();
                            $item->save();
                        }

                        // Update total and status
                        $order->calculate_shipping();
                        $order->calculate_totals();
                        $order->set_status('processing');
                        $order->save();

                        die(wp_json_encode(true));
                    }
                    // Erreurs
                    else {
                        if ($code === 'PHONE')
                            die(wp_json_encode('PHONE'));
                        if ($code === 'MISSING')
                            die(wp_json_encode('MISSING'));
                    }
                }
                else {
                    die(wp_json_encode('DELETE'));
                }
            }
        }
        die(wp_json_encode(false));
    }

    add_action('wp_ajax_lar_resend_order', 'prefix_ajax_lar_resend_order');
}

if (!function_exists('prefix_ajax_lar_cancel_orders')) {
    /**
     * Returns the carriers' quotes.
     * @return void
     */
    function prefix_ajax_lar_cancel_orders() {
        if (isset($_POST['order']) && wp_verify_nonce($_POST['nonce'], 'lar-ajax-nonce-del') &&
            $_POST['order'] && current_user_can('edit_posts')) {
            $order = wc_get_order($_POST['order']);

            if ($order) {
                $old_orders = $order->get_meta('_lar_old_orders');
                $failed = [];

                if ($old_orders) {
                    foreach ($old_orders as $old_order) {
                        $deleted = Ship_Discounts_Carriers::deleteOrder($old_order, $order);
                        if ($deleted === false || (isset($deleted['result']) && $deleted['result'] !== 'success'))
                            $failed[] = $old_order;

                        // TODO: Gérer les cas d'erreur
                    }
                }

                $order->update_meta_data('_lar_old_orders', $failed);
                $order->save();

                die(wp_json_encode($failed));
            }
        }
        die(wp_json_encode(false));
    }

    add_action('wp_ajax_lar_cancel_orders', 'prefix_ajax_lar_cancel_orders');
}